import { defaultTheme, themable } from "../Theme";

const theme = {
  primary: "orange",
  secondary: "blue",
  button: {
    primary: {
      background: "red",
      text: "black",
    },
  },
};

describe("regarding the theme module", () => {
  describe("regarding the themable function", () => {
    describe("given an array of props", () => {
      it("should return the first prop of the array from the theme", () => {
        const props = ["button.primary.background", "primary"];

        expect(themable({ theme, props })).toEqual(
          theme.button.primary.background
        );
      });

      describe("where the first prop is not defined in the theme", () => {
        describe("and the first array value is not undefined", () => {
          it("should return the value", () => {
            const props = ["white", "button.primary.background", "primary"];

            expect(themable({ theme, props })).toEqual("white");
          });
        });

        describe("and the first array value is undefined", () => {
          it("should return the next array value", () => {
            const color = undefined;
            const props = [color, "button.primary.background", "primary"];

            expect(themable({ theme, props })).toEqual("red");
          });
        });
      });

      describe("where the first and the second prop are not defined in the theme", () => {
        it("should return the third array value", () => {
          const props = [
            undefined,
            "button.primary.background.color",
            "secondary",
            "primary",
          ];

          expect(themable({ theme, props })).toEqual("blue");
        });
      });

      describe("where the array only contains one value", () => {
        it("should return the value from the theme", () => {
          const props = ["primary"];

          expect(themable({ theme, props })).toEqual(theme.primary);
        });

        describe("and the value is not in the theme", () => {
          it("should return the given value", () => {
            const props = ["#fff"];

            expect(themable({ theme, props })).toEqual("#fff");
          });
        });

        describe("given no theme", () => {
          it("should return the default theme value", () => {
            const props = ["primary"];

            expect(themable({ props })).toEqual(defaultTheme.primary);
          });

          describe("and the value is not in the default theme", () => {
            it("should return the given value", () => {
              const props = ["#fff"];

              expect(themable({ props })).toEqual("#fff");
            });
          });
        });
      });

      describe("where the prop requested is in the theme but is an object", () => {
        it("should return the whole object", () => {
          const color = undefined;
          const props = [
            color,
            "button.primary.background.color",
            "button",
            "primary",
          ];

          expect(themable({ theme, props })).toEqual({
            primary: {
              background: "red",
              text: "black",
            },
          });
        });
      });
    });

    describe("given a prop separated by points", () => {
      describe("and is a theme prop (words separated by points)", () => {
        it("should return the prop from the theme", () => {
          const props = ["button.primary.background"];

          expect(themable({ theme, props })).toEqual("red");
        });
      });

      describe("and is a css prop that contains points", () => {
        it("should return the prop passed as input", () => {
          const props = ["rgba(255,255,255,0.9)"];

          expect(themable({ theme, props })).toEqual("rgba(255,255,255,0.9)");
        });
      });

      describe("and is not a theme prop", () => {
        it("should return the next prop", () => {
          const props = ["button.primary.color", "#fff"];

          expect(themable({ theme, props })).toEqual("#fff");
        });

        describe("and is not a default theme prop", () => {
          it("should return the next prop", () => {
            const props = ["button.primary.color", "#fff"];

            expect(themable({ props })).toEqual("#fff");
          });
        });
      });
    });

    describe("given a color prop", () => {
      it("should return the prop from the theme", () => {
        const color = "primary";
        const props = [`button.${color}.background`, color];

        expect(themable({ theme, props })).toEqual("red");
      });

      describe("and the color is not defined in the theme", () => {
        it("should return the color prop", () => {
          const color = "white";
          const props = [color];

          expect(themable({ theme, props })).toEqual("white");
        });

        it("should return the color prop", () => {
          const color = "white";
          const props = [`button.${color}.background`, color];

          expect(themable({ theme, props })).toEqual("white");
        });
      });
    });
  });
});
